@@ -258,6 +258,9 @@ display_chassis(struct writer* w, lldpctl_atom_t* chassis, int details)
 	lldpctl_atom_foreach(mgmts, mgmt) {
 		tag_datatag(w, "mgmt-ip", "MgmtIP",
 		    lldpctl_atom_get_str(mgmt, lldpctl_k_mgmt_ip));
+		if (lldpctl_atom_get_int(mgmt, lldpctl_k_mgmt_iface_index))
+			tag_datatag(w, "mgmt-iface", "MgmtIface",
+			    lldpctl_atom_get_str(mgmt, lldpctl_k_mgmt_iface_index));
 	}
 	lldpctl_atom_dec_ref(mgmts);
 
@@ -428,10 +431,13 @@ display_port(struct writer *w, lldpctl_atom_t *port, int details)
 			tag_data(w, lldpctl_atom_get_str(dot3_power,
 				lldpctl_k_dot3_power_devicetype));;
 			tag_end(w);
-			tag_start(w, "pairs", "Power pairs");
-			tag_data(w, lldpctl_atom_get_str(dot3_power,
-				lldpctl_k_dot3_power_pairs));
-			tag_end(w);
+			if (lldpctl_atom_get_int(dot3_power,
+			    lldpctl_k_dot3_power_devicetype) == LLDP_DOT3_POWER_PSE) {
+				tag_start(w, "pairs", "Power pairs");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+				    lldpctl_k_dot3_power_pairs));
+				tag_end(w);
+			}
 			tag_start(w, "class", "Class");
 			tag_data(w, lldpctl_atom_get_str(dot3_power,
 				lldpctl_k_dot3_power_class));
@@ -466,6 +472,63 @@ display_port(struct writer *w, lldpctl_atom_t *port, int details)
 				tag_end(w);
 			}
 
+			/* 802.3bt */
+			if (lldpctl_atom_get_int(dot3_power,
+				lldpctl_k_dot3_power_type_ext) > LLDP_DOT3_POWER_8023BT_OFF) {
+				tag_start(w, "requested-a", "Requested mode A");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_requested_a));
+				tag_end(w);
+				tag_start(w, "requested-b", "Requested mode B");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_requested_b));
+				tag_end(w);
+				tag_start(w, "allocated-a", "Allocated alternative A");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_allocated_a));
+				tag_end(w);
+				tag_start(w, "allocated-b", "Allocated alternative B");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_allocated_b));
+				tag_end(w);
+				tag_start(w, "pse-powering-status", "PSE powering status");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_pse_status));
+				tag_end(w);
+				tag_start(w, "pd-powering-status", "PD powering status");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_pd_status));
+				tag_end(w);
+				tag_start(w, "power-pairs-ext", "Power pairs extra");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_pse_pairs_ext));
+				tag_end(w);
+				tag_start(w, "power-class-ext-a", "Class extra A");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_class_a));
+				tag_end(w);
+				tag_start(w, "power-class-ext-b", "Class extra B");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_class_b));
+				tag_end(w);
+				tag_start(w, "power-class-ext", "Class extra");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_class_ext));
+				tag_end(w);
+				tag_start(w, "power-type-ext", "Power type extra");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_type_ext));
+				tag_end(w);
+				tag_start(w, "pd-load", "PD load");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_pd_load));
+				tag_end(w);
+				tag_start(w, "max-power", "PSE maximum available power");
+				tag_data(w, lldpctl_atom_get_str(dot3_power,
+					lldpctl_k_dot3_power_pse_max));
+				tag_end(w);
+			}
+
 			tag_end(w);
 		}
 		lldpctl_atom_dec_ref(dot3_power);
@@ -490,9 +553,11 @@ display_local_ttl(struct writer *w, lldpctl_conn_t *conn, int details)
 	}
 
 	tx_hold = lldpctl_atom_get_int(configuration, lldpctl_k_config_tx_hold);
-	tx_interval = lldpctl_atom_get_int(configuration, lldpctl_k_config_tx_interval);
+	tx_interval = lldpctl_atom_get_int(configuration, lldpctl_k_config_tx_interval_ms);
 
-	if (asprintf(&ttl, "%lu", tx_hold*tx_interval) == -1) {
+	tx_interval = (tx_interval * tx_hold + 999) / 1000;
+
+	if (asprintf(&ttl, "%lu", tx_interval) == -1) {
 		log_warnx("lldpctl", "not enough memory to build TTL.");
 		goto end;
 	}
@@ -519,14 +584,16 @@ display_vlans(struct writer *w, lldpctl_atom_t *port)
 	lldpctl_atom_foreach(vlans, vlan) {
 		vid = lldpctl_atom_get_int(vlan,
 		    lldpctl_k_vlan_id);
-		if (pvid == vid)
-			foundpvid = 1;
 
 		tag_start(w, "vlan", "VLAN");
 		tag_attr(w, "vlan-id", "",
 		    lldpctl_atom_get_str(vlan, lldpctl_k_vlan_id));
-		if (pvid == vid)
+		if (pvid == vid) {
 			tag_attr(w, "pvid", "pvid", "yes");
+			foundpvid = 1;
+		} else {
+			tag_attr(w, "pvid", "pvid", "no");
+		}
 		tag_data(w, lldpctl_atom_get_str(vlan,
 			lldpctl_k_vlan_name));
 		tag_end(w);
@@ -668,8 +735,8 @@ display_interface(lldpctl_conn_t *conn, struct writer *w, int hidden,
  *
  * @param conn       Connection to lldpd.
  * @param w          Writer.
- * @param hidden     Whatever to show hidden ports.
  * @param env        Environment from which we may find the list of ports.
+ * @param hidden     Whatever to show hidden ports.
  * @param details    Level of details we need (DISPLAY_*).
  */
 void
@@ -884,8 +951,12 @@ display_configuration(lldpctl_conn_t *conn, struct writer *w)
 
 	tag_datatag(w, "tx-delay", "Transmit delay",
 	    lldpctl_atom_get_str(configuration, lldpctl_k_config_tx_interval));
+	tag_datatag(w, "tx-delay-ms", "Transmit delay in milliseconds",
+	    lldpctl_atom_get_str(configuration, lldpctl_k_config_tx_interval_ms));
 	tag_datatag(w, "tx-hold", "Transmit hold",
 	    lldpctl_atom_get_str(configuration, lldpctl_k_config_tx_hold));
+	tag_datatag(w, "max-neighbors", "Maximum number of neighbors",
+	    lldpctl_atom_get_str(configuration, lldpctl_k_config_max_neighbors));
 	tag_datatag(w, "rx-only", "Receive mode",
 	    lldpctl_atom_get_int(configuration, lldpctl_k_config_receiveonly)?
 	    "yes":"no");
@@ -893,8 +964,12 @@ display_configuration(lldpctl_conn_t *conn, struct writer *w)
 	    N(lldpctl_atom_get_str(configuration, lldpctl_k_config_mgmt_pattern)));
 	tag_datatag(w, "iface-pattern", "Interface pattern",
 	    N(lldpctl_atom_get_str(configuration, lldpctl_k_config_iface_pattern)));
+	tag_datatag(w, "perm-iface-pattern", "Permanent interface pattern",
+	    N(lldpctl_atom_get_str(configuration, lldpctl_k_config_perm_iface_pattern)));
 	tag_datatag(w, "cid-pattern", "Interface pattern for chassis ID",
 	    N(lldpctl_atom_get_str(configuration, lldpctl_k_config_cid_pattern)));
+	tag_datatag(w, "cid-string", "Override chassis ID with",
+	    N(lldpctl_atom_get_str(configuration, lldpctl_k_config_cid_string)));
 	tag_datatag(w, "description", "Override description with",
 	    N(lldpctl_atom_get_str(configuration, lldpctl_k_config_description)));
 	tag_datatag(w, "platform", "Override platform with",
