@@ -639,10 +639,10 @@ agent_h_scalars(struct variable *vp, oid *name, size_t *length,
 
 	switch (vp->magic) {
 	case LLDP_SNMP_TXINTERVAL:
-                long_ret = scfg->g_config.c_tx_interval;
+                long_ret = (scfg->g_config.c_tx_interval+999) / 1000;
 		return (u_char *)&long_ret;
 	case LLDP_SNMP_TXMULTIPLIER:
-		long_ret = scfg->g_config.c_ttl / scfg->g_config.c_tx_interval;
+		long_ret = scfg->g_config.c_tx_hold;
 		return (u_char *)&long_ret;
 	case LLDP_SNMP_REINITDELAY:
 		long_ret = 1;
