@@ -307,7 +307,7 @@ ifbsd_check_vlan(struct lldpd *cfg,
 	    "%s is VLAN %d of %s",
 	    vlan->name, vreq.vlr_tag, lower->name);
 	vlan->lower = lower;
-	vlan->vlanid = vreq.vlr_tag;
+	bitmap_set(vlan->vlan_bmap, vreq.vlr_tag);
 	vlan->type |= IFACE_VLAN_T;
 }
 
@@ -672,7 +672,7 @@ interfaces_update(struct lldpd *cfg)
 #ifdef ENABLE_DOT1
 	interfaces_helper_vlan(cfg, interfaces);
 #endif
-	interfaces_helper_mgmt(cfg, addresses);
+	interfaces_helper_mgmt(cfg, addresses, interfaces);
 	interfaces_helper_chassis(cfg, interfaces);
 
 	/* Mac/PHY */
