@@ -25,12 +25,16 @@
 #include <net/if_arp.h>
 #include <linux/netlink.h>
 #include <linux/rtnetlink.h>
+#include <linux/if_bridge.h>
 
 #define NETLINK_BUFFER 4096
 
 struct netlink_req {
 	struct nlmsghdr hdr;
-	struct rtgenmsg gen;
+	struct ifinfomsg ifm;
+	/* attribute has to be NLMSG aligned */
+	struct rtattr ext_req __attribute__ ((aligned(NLMSG_ALIGNTO)));
+	__u32 ext_filter_mask;
 };
 
 struct lldpd_netlink {
@@ -41,7 +45,6 @@ struct lldpd_netlink {
 	struct interfaces_address_list *addresses;
 };
 
-
 /**
  * Set netlink socket buffer size.
  *
@@ -141,12 +144,12 @@ netlink_send(int s, int type, int family, int seq)
 {
 	struct netlink_req req = {
 		.hdr = {
-			.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtgenmsg)),
+			.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifinfomsg)),
 			.nlmsg_type = type,
 			.nlmsg_flags = NLM_F_REQUEST | NLM_F_DUMP,
 			.nlmsg_seq = seq,
 			.nlmsg_pid = getpid() },
-		.gen = { .rtgen_family = family }
+		.ifm = { .ifi_family = family }
 	};
 	struct iovec iov = {
 		.iov_base = &req,
@@ -160,6 +163,16 @@ netlink_send(int s, int type, int family, int seq)
 		.msg_namelen = sizeof(struct sockaddr_nl)
 	};
 
+	if (family == AF_BRIDGE) {
+		unsigned int len = RTA_LENGTH(sizeof(__u32));
+		/* request bridge vlan attributes */
+		req.ext_req.rta_type = IFLA_EXT_MASK;
+		req.ext_req.rta_len = len;
+		req.ext_filter_mask = RTEXT_FILTER_BRVLAN;
+		req.hdr.nlmsg_len = NLMSG_ALIGN(req.hdr.nlmsg_len) + RTA_ALIGN(len);
+		iov.iov_len = req.hdr.nlmsg_len;
+	}
+
 	/* Send netlink message. This is synchronous but we are guaranteed
 	 * to not block. */
 	log_debug("netlink", "sending netlink message");
@@ -193,6 +206,7 @@ netlink_parse_linkinfo(struct interfaces_device *iff, struct rtattr *rta, int le
 {
 	struct rtattr *link_info_attrs[IFLA_INFO_MAX+1] = {};
 	char *kind = NULL;
+	uint16_t vlan_id;
 
 	netlink_parse_rtattr(link_info_attrs, IFLA_INFO_MAX, rta, len);
 
@@ -226,15 +240,66 @@ netlink_parse_linkinfo(struct interfaces_device *iff, struct rtattr *rta, int le
 		    RTA_PAYLOAD(link_info_attrs[IFLA_INFO_DATA]));
 
 		if (vlan_link_info_data_attrs[IFLA_VLAN_ID]) {
-			iff->vlanid = *(uint16_t *)RTA_DATA(vlan_link_info_data_attrs[IFLA_VLAN_ID]);
+			vlan_id = *(uint16_t *)RTA_DATA(vlan_link_info_data_attrs[IFLA_VLAN_ID]);
+			bitmap_set(iff->vlan_bmap, vlan_id);
 			log_debug("netlink", "VLAN ID for interface %s is %d",
-			    iff->name, iff->vlanid);
+			    iff->name, vlan_id);
+		}
+	}
+
+	if (kind && !strcmp(kind, "bridge") && link_info_attrs[IFLA_INFO_DATA]) {
+		struct rtattr *bridge_link_info_data_attrs[IFLA_BR_MAX+1] = {};
+		netlink_parse_rtattr(bridge_link_info_data_attrs, IFLA_BR_MAX,
+		    RTA_DATA(link_info_attrs[IFLA_INFO_DATA]),
+		    RTA_PAYLOAD(link_info_attrs[IFLA_INFO_DATA]));
+
+		if (bridge_link_info_data_attrs[IFLA_BR_VLAN_FILTERING] &&
+		    *(uint8_t *)RTA_DATA(bridge_link_info_data_attrs[IFLA_BR_VLAN_FILTERING]) > 0) {
+			iff->type |= IFACE_BRIDGE_VLAN_T;
 		}
 	}
 
 	free(kind);
 }
 
+/**
+ * Parse a `afspec` attributes.
+ *
+ * @param iff where to put the result
+ * @param rta afspec attribute
+ * @param len length of attributes
+ */
+static void
+netlink_parse_afspec(struct interfaces_device *iff, struct rtattr *rta, int len)
+{
+	while (RTA_OK(rta, len)) {
+		struct bridge_vlan_info *vinfo;
+		switch (rta->rta_type) {
+		case IFLA_BRIDGE_VLAN_INFO:
+			vinfo = RTA_DATA(rta);
+			log_debug("netlink", "found VLAN %d on interface %s",
+			    vinfo->vid, iff->name ? iff->name : "(unknown)");
+
+			bitmap_set(iff->vlan_bmap, vinfo->vid);
+			if (vinfo->flags & (BRIDGE_VLAN_INFO_PVID | BRIDGE_VLAN_INFO_UNTAGGED))
+				iff->pvid = vinfo->vid;
+			break;
+		default:
+			log_debug("netlink", "unknown afspec attribute type %d for iface %s",
+			    rta->rta_type, iff->name ? iff->name : "(unknown)");
+			break;
+		}
+		rta = RTA_NEXT(rta, len);
+	}
+	/* All enbridged interfaces will have VLAN 1 by default, ignore it */
+	if (iff->vlan_bmap[0] == 2 && (bitmap_numbits(iff->vlan_bmap) == 1)
+			&& iff->pvid == 1) {
+		log_debug("netlink", "found only default VLAN 1 on interface %s, removing",
+		    iff->name ? iff->name : "(unknown)");
+		iff->vlan_bmap[0] = iff->pvid = 0;
+	}
+}
+
 /**
  * Parse a `link` netlink message.
  *
@@ -283,13 +348,18 @@ netlink_parse_link(struct nlmsghdr *msg,
 			break;
 		case IFLA_LINK:
 			/* Index of "lower" interface */
-			iff->lower_idx = *(int*)RTA_DATA(attribute);
-			log_debug("netlink", "attribute IFLA_LINK for %s: %d",
-			    iff->name ? iff->name : "(unknown)", iff->lower_idx);
+			if (iff->lower_idx == -1) {
+				iff->lower_idx = *(int*)RTA_DATA(attribute);
+				log_debug("netlink", "attribute IFLA_LINK for %s: %d",
+				    iff->name ? iff->name : "(unknown)", iff->lower_idx);
+			} else {
+				log_debug("netlink", "attribute IFLA_LINK for %s: %d (ignored)",
+				    iff->name ? iff->name : "(unknown)", iff->lower_idx);
+			}
 			break;
 		case IFLA_LINK_NETNSID:
 			/* Is the lower interface into another namesapce? */
-			iff->lower_idx = -1;
+			iff->lower_idx = -2;
 			log_debug("netlink", "attribute IFLA_LINK_NETNSID received for %s",
 			    iff->name ? iff->name : "(unknown)");
 			break;
@@ -304,6 +374,10 @@ netlink_parse_link(struct nlmsghdr *msg,
 		case IFLA_LINKINFO:
 			netlink_parse_linkinfo(iff, RTA_DATA(attribute), RTA_PAYLOAD(attribute));
 			break;
+		case IFLA_AF_SPEC:
+			if (ifi->ifi_family != AF_BRIDGE) break;
+			netlink_parse_afspec(iff, RTA_DATA(attribute), RTA_PAYLOAD(attribute));
+			break;
 		default:
 			log_debug("netlink", "unhandled link attribute type %d for iface %s",
 			    attribute->rta_type, iff->name ? iff->name : "(unknown)");
@@ -321,16 +395,14 @@ netlink_parse_link(struct nlmsghdr *msg,
 		 * and we don't want to miss it. */
 		iff->flags &= ~IFF_SLAVE;
 	}
+	if (iff->lower_idx == -2)
+		iff->lower_idx = -1;
 
 	if (ifi->ifi_family == AF_BRIDGE && msg->nlmsg_type == RTM_DELLINK && iff->upper_idx != -1) {
 		log_debug("netlink", "removal of %s from bridge %d",
 		    iff->name, iff->upper_idx);
 		msg->nlmsg_type = RTM_NEWLINK;
 		iff->upper_idx = -1;
-	} else if (ifi->ifi_family != 0) {
-		log_debug("netlink", "skip non-generic message update %d at index %d",
-		    ifi->ifi_type, ifi->ifi_index);
-		return -1;
 	}
 
 	log_debug("netlink", "parsed link %d (%s, flags: %d)",
@@ -423,8 +495,10 @@ netlink_merge(struct interfaces_device *old, struct interfaces_device *new)
 		new->mtu = old->mtu;
 	if (new->type == 0)
 		new->type = old->type;
-	if (new->vlanid == 0)
-		new->vlanid = old->vlanid;
+
+	if (bitmap_isempty(new->vlan_bmap) && new->type == IFACE_VLAN_T)
+		memcpy((void *)new->vlan_bmap, (void *)old->vlan_bmap,
+				sizeof(uint32_t) * VLAN_BITMAP_LEN);
 
 	/* It's not possible for lower link to change */
 	new->lower_idx = old->lower_idx;
@@ -442,8 +516,7 @@ netlink_recv(struct lldpd *cfg,
     struct interfaces_device_list *ifs,
     struct interfaces_address_list *ifas)
 {
-	int end = 0, ret = 0;
-	int flags = MSG_PEEK | MSG_TRUNC;
+	int end = 0, ret = 0, flags, retry = 0;
 	struct iovec iov;
 	int link_update = 0;
 	int s = cfg->g_netlink->nl_socket;
@@ -471,12 +544,16 @@ netlink_recv(struct lldpd *cfg,
 			.msg_name = &peer,
 			.msg_namelen = sizeof(struct sockaddr_nl)
 		};
-
+		flags = MSG_PEEK | MSG_TRUNC;
 retry:
 		len = recvmsg(s, &rtnl_reply, flags);
 		if (len == -1) {
 			if (errno == EAGAIN || errno == EWOULDBLOCK) {
-				log_debug("netlink", "should have received something, but didn't");
+				if (retry++ == 0) {
+					levent_recv_error(s, "netlink socket");
+					goto retry;
+				}
+				log_warnx("netlink", "should have received something, but didn't");
 				ret = 0;
 				goto out;
 			}
@@ -650,7 +727,6 @@ retry:
 				    msg->nlmsg_type, msg->nlmsg_len);
 			}
 		}
-		flags = MSG_PEEK | MSG_TRUNC;
 	}
 end:
 	if (link_update) {
@@ -686,8 +762,11 @@ end:
 						if (iface2->lower_idx == iface1->index) {
 							iface1->lower = NULL;
 							log_debug("netlink",
-							    "link loop detected between %s and %s",
-							    iface1->name, iface2->name);
+							    "link loop detected between %s(%d) and %s(%d)",
+							    iface1->name,
+							    iface1->index,
+							    iface2->name,
+							    iface2->index);
 						} else {
 							log_debug("netlink",
 							    "lower interface for %s is %s",
@@ -757,6 +836,10 @@ netlink_change_cb(struct lldpd *cfg)
 static int
 netlink_initialize(struct lldpd *cfg)
 {
+#ifdef ENABLE_DOT1
+	struct interfaces_device *iff;
+#endif
+
 	if (cfg->g_netlink) return 0;
 
 	log_debug("netlink", "initialize netlink subsystem");
@@ -792,6 +875,18 @@ netlink_initialize(struct lldpd *cfg)
 	if (netlink_send(cfg->g_netlink->nl_socket, RTM_GETLINK, AF_PACKET, 2) == -1)
 		goto end;
 	netlink_recv(cfg, ifs, NULL);
+#ifdef ENABLE_DOT1
+	/* If we have a bridge, search for VLAN-aware bridges */
+	TAILQ_FOREACH(iff, ifs, next) {
+		if (iff->type & IFACE_BRIDGE_T) {
+			log_debug("netlink", "interface %s is a bridge, check for VLANs", iff->name);
+			if (netlink_send(cfg->g_netlink->nl_socket, RTM_GETLINK, AF_BRIDGE, 3) == -1)
+				goto end;
+			netlink_recv(cfg, ifs, NULL);
+			break;
+		}
+	}
+#endif
 
 	/* Listen to any future change */
 	cfg->g_iface_cb = netlink_change_cb;
