@@ -125,6 +125,22 @@ struct lldpd_dot3_power {
 	u_int8_t		priority;
 	u_int16_t		requested;
 	u_int16_t		allocated;
+
+	/* For 802.3BT */
+	u_int8_t		pd_4pid;
+	u_int16_t		requested_a;
+	u_int16_t		requested_b;
+	u_int16_t		allocated_a;
+	u_int16_t		allocated_b;
+	u_int16_t		pse_status;
+	u_int8_t		pd_status;
+	u_int8_t		pse_pairs_ext;
+	u_int8_t		class_a;
+	u_int8_t		class_b;
+	u_int8_t		class_ext;
+	u_int8_t		type_ext;
+	u_int8_t		pd_load;
+	u_int16_t		pse_max;
 };
 MARSHAL(lldpd_dot3_power);
 #endif
@@ -244,9 +260,9 @@ struct lldpd_port {
 	struct lldpd_frame	*p_lastframe;  /* Frame received during last update */
 	u_int8_t		 p_protocol;   /* Protocol used to get this port */
 	u_int8_t		 p_hidden_in:1; /* Considered as hidden for reception */
-	u_int8_t		 p_hidden_out:2; /* Considered as hidden for emission */
-	u_int8_t		 p_disable_rx:3; /* Should RX be disabled for this port? */
-	u_int8_t		 p_disable_tx:4; /* Should TX be disabled for this port? */
+	u_int8_t		 p_hidden_out:1; /* Considered as hidden for emission */
+	u_int8_t		 p_disable_rx:1; /* Should RX be disabled for this port? */
+	u_int8_t		 p_disable_tx:1; /* Should TX be disabled for this port? */
 	/* Important: all fields that should be ignored to check if a port has
 	 * been changed should be before this mark. */
 #define LLDPD_PORT_START_MARKER (offsetof(struct lldpd_port, _p_hardware_flags))
@@ -372,7 +388,7 @@ MARSHAL_END(lldpd_port_set);
 
 struct lldpd_config {
 	int c_paused;	        /* lldpd is paused */
-	int c_tx_interval;	/* Transmit interval */
+	int c_tx_interval;	/* Transmit interval (in ms) */
 	int c_ttl;		/* TTL */
 	int c_smart;		/* Bitmask for smart configuration (see SMART_*) */
 	int c_receiveonly;	/* Receive only mode */
